# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.2.0

import enum

import winrt.system
from winrt._winrt_windows_system_diagnostics_deviceportal import (
    DevicePortalConnection,
    DevicePortalConnectionClosedEventArgs,
    DevicePortalConnectionRequestReceivedEventArgs,
)

__all__ = [
    "DevicePortalConnectionClosedReason",
    "DevicePortalConnection",
    "DevicePortalConnectionClosedEventArgs",
    "DevicePortalConnectionRequestReceivedEventArgs",
]

class DevicePortalConnectionClosedReason(enum.IntEnum):
    UNKNOWN = 0
    RESOURCE_LIMITS_EXCEEDED = 1
    PROTOCOL_ERROR = 2
    NOT_AUTHORIZED = 3
    USER_NOT_PRESENT = 4
    SERVICE_TERMINATED = 5

